<?php


namespace app\cbcrm\services;


use app\cbcrm\model\config\SettingModel;
use app\cbcrm\model\copyright\OrderModel;
use app\cbcrm\model\member\MemberModel;
use EasyWeChat\Factory;
use EasyWeChat\Kernel\Exceptions\InvalidArgumentException;
use EasyWeChat\Kernel\Exceptions\InvalidConfigException;
use GuzzleHttp\Exception\GuzzleException;
use think\Exception;
use think\facade\Log;

class PayService
{
    /**
     * @title 微信支付
     * @param float $amount 订单金额
     * @param string $order_no 订单号
     * @param string $goods_name 商品名称
     * @param string $platform 支付平台 wechat 公众号   mpWechat 微信小程序
     * @param integer $wid 账户ID
     * @param string $notify_url 回调地址 ''
     * */
    public function payWechat($amount, $order_no, $goods_name,$wid,$notify_url)
    {
        $config = (new SettingModel())->getConfigGroup('wechat', $wid);
        $payConfig = [
            // 必要配置
            'app_id' => $config['app_id'],
            'mch_id' => $config['mch_id'],
            'key' => $config['api_key'],   // API v2 密钥 (注意: 是v2密钥 是v2密钥 是v2密钥)
            'notify_url' => \request()->domain() . '/cbcrm/api.notify'.$notify_url,
        ];
        $app = Factory::payment($payConfig);
        $payData = null;
        try {
            $payData = $app->order->unify([
                'body' => $goods_name,
                'out_trade_no' => $order_no,
                'total_fee' => $amount * 100, // 单位：分
                'trade_type' => 'NATIVE', // 请对应换成你的支付方式对应的值类型
            ]);
            if (isset($payData['return_code']) && $payData['return_code'] === 'FAIL') {
                throw new Exception($payData['return_msg']);
            }
            if (isset($payData['result_code']) && $payData['result_code'] === 'FAIL') {
                throw new Exception($payData['err_code_des']);
            }

        } catch (Exception|InvalidArgumentException|InvalidConfigException|GuzzleException $e) {
            Log::error($e->getMessage());
            return null;
        }
        return $payData;
    }


    /**
     * @title 余额支付
     * @param float $amount 订单金额
     * @param string $order_no 订单号
     * @param integer $wid 账户ID
     * @param object $auth 登录用户
     * @param object $orderClass 订单类名
     * @param string $sms_type 短信通知产品类型
     * */
    public function payBalance($amount, $order_no,$wid,$auth,$orderClass,$sms_type = '商品')
    {
        if ($auth->balance < $amount) {
            return '可用余额不足';
        }
        $orderService = app()->make($orderClass);
        $order = $orderService->where('order_no', $order_no)->find();
        if (!$order || $order->pay_status != 0) {
            return '待支付订单不存在';
        }

        $userInfo = (new MemberModel())->where('id', $auth->id)->find();
        // 扣除用户余额
        $userInfo->balance = $userInfo->balance - $amount;

        $res = $userInfo->save();

        if ($res) {
            // 发送短信通知
            $smsServices = app()->make(SmsServices::class);
            $mobile = (new SettingModel())->getConfigValue('sms','acceptPhone', $wid);
            $smsServices->send($mobile,['type'=>$sms_type,'order_no'=>$order_no],'后台新订单',$wid);
            return true;
        } else {
            return false;
        }
    }

    /**
     * @title 微信支付退款
     * @param string $order_no 订单号
     * @param string $refundNumber 退款单号
     * @param integer $totalFee 订单金额
     * @param integer $refundFee 退款金额
     * @param integer $wid 账户ID
     * */
    public function refundWechat(string $order_no, string $refundNumber, int $totalFee, int $refundFee,int $wid,string $refundDesc)
    {
        $config = (new SettingModel())->getConfigGroup('wechat', $wid);
        $payConfig = [
            // 必要配置
            'app_id' => $config['app_id'],
            'mch_id' => $config['mch_id'],
            'key' => $config['api_key'],   // API v2 密钥 (注意: 是v2密钥 是v2密钥 是v2密钥)
            'cert_path'=>'',
            'key_path'=>''
        ];
        $app = Factory::payment($payConfig);
        try {
            $result = $app->refund->byOutTradeNumber($order_no, $refundNumber, $totalFee, $refundFee,[
                'refund_desc'=>$refundDesc
            ]);
            Log::error($result);
        } catch (Exception|InvalidConfigException $e) {
        }
    }

    /**
     * @title 余额支付退款
     * @param string $order_no 订单号
     * @param string $refundNumber 退款单号
     * @param integer $totalFee 订单金额
     * @param integer $refundFee 退款金额
     * @param integer $wid 账户ID
     * */
    public function refundBalance(string $order_no, string $refundNumber, int $totalFee, int $refundFee,int $wid,string $refundDesc)
    {

    }
}
